#include "ARMCFunctions.h"
#include <math.h>


inline unsigned int HueToRGB(double H)
{
  float HS;
  int i,R,G,B,F;
  F = (int)(H / 360.0);
  H = H - 360.0 * F;
  if (H < 0.00)
    H = 360.0 + H;
  HS = H / 60.0;
  i = (int)HS;

  switch (i)
  {
    case 0: 
      F = (HS - i) * 63;
      R = 31; G = F; B = 0;
      break;
    case 1:
      F = (HS - i) * 31;
      R = 31-F; G = 63; B = 0;
      break;
    case 2:
      F = (HS - i) * 31;
      R = 0; G = 63 ; B = F;
      break;
    case 3:
      F = (HS - i)*63;
      R = 0; G = 63-F; B = 31;
      break;
    case 4:
      F = (HS - i)*31;
      R = F; G = 0; B = 31;
      break;
    case 5:
      F = (HS - i)*31;
      R = 31; G = 0; B = 31-F;
      break;
  }
  return ((R << 11) | (G  << 5) | B);
}


inline float fastlogf (float a)

{
    
  float m, r, s, t, i, f;
    
  unsigned int e,q;

    
  q = *((int*)&a);
  e = (q - 0x3f2aaaab) & 0xff800000;
    
  q = q - e;
  m = *((float*)&q);
    
  i = (float)e * 1.19209290e-7f; // 0x1.0p-23
    
  /* m in [2/3, 4/3] */
    
  f = m - 1.0f;
    
  s = f * f;
    
  /* Compute log1p(f) for f in [-1/3, 1/3] */
    
  r = fmaf (0.230836749f, f, -0.279208571f); // 0x1.d8c0f0p-3, -0x1.1de8dap-2
    
  t = fmaf (0.331826031f, f, -0.498910338f); // 0x1.53ca34p-2, -0x1.fee25ap-2
    
  r = fmaf (r, s, t);
    
  r = fmaf (r, s, f);
    
  r = fmaf (i, 0.693147182f, r); // 0x1.62e430p-1 // log(2) 
    
  
  return r;

}


void mandelbrot16(int *itermax,MMFLOAT *magnify, MMFLOAT *xcenter, MMFLOAT *ycenter, MMFLOAT *hstart, MMFLOAT *hmult)
{
   double xsqr,ysqr,x,y,cx,cy,m;
   int iteration,hx,hy;
   unsigned short* page = (unsigned short*)GetPageAddress(0);
   
   int hxres = HRes;        /* horizonal resolution        */
   int hyres = VRes;        /* vertical resolution        */

   for (hy=hyres;hy>=1;hy--)  
     {
       cy = (((float)hy)/((float)hyres)-0.5) / *magnify * 2.5 - *ycenter;

       for (hx=hxres;hx>=1;hx--)  
          {
//           if (!(hx % 64))
             RoutineChecks();
           cx = (((float)hx)/((float)hxres)-0.5) / *magnify * 3.0 + *xcenter;

           
           x = 0.0; y = 0.0;

           for (iteration=1;iteration<*itermax;iteration++)  
             {
               xsqr = x*x;
               ysqr = y*y;

               if (xsqr+ysqr > 4.0)
                  break;

               y = 2.0*x*y+cy;
               x = xsqr-ysqr+cx;

             }

           if (iteration == *itermax)
//             DrawPixel(hx-1,hy-1,0x00);
               *(page + hxres*(hy-1) + hx-1) = 0x00;  
           else
             {
//               m = (double)iteration - (log(log(sqrt(xsqr+ysqr)))) / log(2.0); 
                 m = (float)iteration - (fastlogf(fastlogf(sqrtf(xsqr+ysqr)))) / 0.693147180559945f; 

                 *(page + hxres*(hy-1) + hx-1) = HueToRGB(m * *hmult + *hstart);
//               DrawPixel(hx-1,hy-1,HueToRGB(m * *hmult + *hstart));           
             }
          }
     }
}

